@@ -2,6 +2,7 @@ import os
 from torch.utils.tensorboard import SummaryWriter
 
 from aitoolbox.torchtrain.callbacks.abstract import AbstractExperimentCallback
+from aitoolbox.torchtrain.multi_loss_optim import MultiLoss
 from aitoolbox.experiment.local_save.folder_create import ExperimentFolder as FolderCreator
 from aitoolbox.cloud import s3_available_options, gcs_available_options
 from aitoolbox.cloud.AWS.results_save import BaseResultsSaver as BaseResultsS3Saver
@@ -63,7 +64,7 @@ class TensorboardReporterBaseCB(AbstractExperimentCallback):
         last_batch_loss = self.train_loop_obj.parse_loss(self.train_loop_obj.loss_batch_accum[-1:])
         accum_mean_batch_loss = self.train_loop_obj.parse_loss(self.train_loop_obj.loss_batch_accum)
 
-        if not isinstance(last_batch_loss, dict) and not isinstance(accum_mean_batch_loss, dict):
+        if not isinstance(last_batch_loss, MultiLoss) and not isinstance(accum_mean_batch_loss, MultiLoss):
             last_batch_loss = {'loss': last_batch_loss}
             accum_mean_batch_loss = {'loss': accum_mean_batch_loss}
 
